//////////////////////////////
// environment.wdl
//////////////////////////////
//////////////////////////////
// Winter Contest Project v2.01a, (C) 04, phosphor games, Johannes Bausch
//////////////////////////////
//////////////////////////////
// Umgebung
//////////////////////////////

bind <1.hmp>;

//////////////////////////////
// Framerate
//////////////////////////////
ifdef develop;
	var framerate;

	starter _framerate
	{
		framerate = (1 / (time/16));
		
		while (1)
		{
			framerate = ((framerate * 10) + (1 / (time/16))) / 11;
			wait(1);
		}
	}

	//Framerate Panel
	panel pFramerate
	{
		//	bmap = blendmap2;
		pos_x = 0;
		pos_Y = 0;
		layer = 20;
		
		digits = 10, 10, 3.2, ftDebug, 1, framerate;
		
		flags = d3d, refresh, visible;
	}
endif;

//////////////////////////////
// Vegetation / Steine
//////////////////////////////
var max_entities = 5100;
var max_loops = 120000;

var mip_sprites = 1;

define VEG_GRASS_MAX		4000;
define MODE_COVERALL		1;
define MODE_CROWDS			2;

define VEG_STONE_MAX		50;

define VEG_WOOD_MAX			100;

string strGrass1 = <grass1.tga>;
string strStone1 = <stone.mdl>;
string strWood1 = <wood.mdl>;

var grass_count;
var stone_count;
var wood_count;

action aGrass1
{
	my.facing = off;
	my.oriented = on;
	
	my.scale_x = 0.1 + random(0.3);
	my.scale_y = 0.1 + random(0.3);
	my.pan = random(360);
	
	my.z += 7;

	my.transparent = on;
	my.ambient = -45 - random(20);
	my.alpha = 0;
	
	//Einblenden
	while (my.alpha < 100)
	{
		my.alpha += 20 * time;
		wait(1);
	}
}

entity* curr_ent;
function add_grass(n, _mode)
{
	//Anzahl der neu hinzuzurechnender Grassprites
	var count;
	var count_max;
	
	count_max = min(n, VEG_GRASS_MAX - grass_count);
	
	//Kein Gras hinzuzufgen?
	if (count_max <= 0)
	{
		return;
	}
	
	//Neues Gras
	var neigung;

	trace_mode = ignore_sprites + ignore_passents + ignore_passable;

	//Modus 1: Regelmige Deckung
	if (_mode == MODE_COVERALL)
	{
		while (count < count_max)
		{
			//Position
			temp.x = -4700 + random(9400);
			temp.y = -4700 + random(9400);
			temp.z = 2000;
			
			//Auf Boden Tracen
			trace(temp.x, vector(temp.x, temp.y, -2000));
			
			//Neigung berechnen
			if (normal.z != 0)
			{
				neigung = atan( sqrt(normal.x*normal.x + normal.y*normal.y) / normal.z );
			}

			//Neigung & Hhe
			if ( ((normal.z == 0) || (neigung < 20)) && (target.z < 450) )
			{
				//Grassprite erstellen
				curr_ent = ent_create(strGrass1, target.x, aGrass1);
				
				//Graszhler weiter
				grass_count += 1;
				count += 1;
			}
			
		}
		
		return;
	}
	
	if (_mode == MODE_CROWDS)
	{
		var crowds;
		crowds = int(sqrt(count_max));
		var crowds_count;
		var grass_in_crowd;
		
		while (crowds_count < crowds)
		{
			grass_in_crowd =  (count_max / crowds - 10 + random(20));

			//Position des Grasfeldes
			temp.x = -4400 + random(8800);
			temp.y = -4400 + random(8800);
			temp.z = 2000;
			
			while (count < grass_in_crowd)
			{
				//Position des Grassprites
				temp.x += -100 + random(200);
				temp.y += -100 + random(200);
				
				//Auf Boden Tracen
				trace(temp.x, vector(temp.x, temp.y, -2000));
				
				//Neigung berechnen
				if (normal.z != 0)
				{
					neigung = atan( sqrt(normal.x*normal.x + normal.y*normal.y) / normal.z );
				}
				
				//Neigung & Hhe
				if ( ((normal.z == 0) || (neigung < 20)) && (target.z < 450) )
				{
					//Grassprite erstellen
					curr_ent = ent_create(strGrass1, target.x, aGrass1);
					
					//Graszhler weiter
					grass_count += 1;
					count += 1;
				}
			}
			
			count = 0;
			crowds_count += 1;
		}
		
		return;
	}
}

//////////////////////////////////////////
var shadow_stencil = on;
var shadow_range = 100;

material mat_wood1
{
	diffuse_red = 200;
	diffuse_green = 200;
	diffuse_blue = 200;
	
	ambient_red = 0;
	ambient_green = 0;
	ambient_blue = 0;
}

action aWood1
{
	my.pan = random(360);
	//	my.tilt = random(360);
	my.tilt = -20 + random(40);
	
	my.shadow = on;
	
	vec_scale(my.scale_x, random(0.5) + 0.5);
	
	//	my.z += my.scale_x * my.scale_x;
	
	my.material = mat_wood1;
}

function add_wood(n)
{
	var count;
	var count_max;
	
	count_max = min(n, VEG_WOOD_MAX - wood_count);
	
	var neigung;
	
	if (count_max <= 0)
	{
		return;
	}
	
	trace_mode = ignore_sprites + ignore_passents + ignore_passable;

	while (count < count_max)
	{
		//Position
		temp.x = -4700 + random(9400);
		temp.y = -4700 + random(9400);
		temp.z = 2000;
		
		//Auf Boden Tracen
		trace(temp.x, vector(temp.x, temp.y, -2000));
		
		//Neigung berechnen
		if (normal.z != 0)
		{
			neigung = atan( sqrt(normal.x*normal.x + normal.y*normal.y) / normal.z );
		}
		
		//Neigung & Hhe
		if ( ((normal.z == 0) || (neigung < 40)) && (target.z < 450) )
		{
			//Grassprite erstellen
			ent_create(strWood1, target.x, aWood1);
			
			//Graszhler weiter
			wood_count += 1;
			count += 1;
		}
		
	}
}

////////////////////////
material mat_stone1
{
	diffuse_red = 200;
	diffuse_green = 200;
	diffuse_blue = 200;
	
	ambient_red = 0;
	ambient_green = 0;
	ambient_blue = 0;
}

action aStone1
{
	my.pan = random(360);
	//	my.tilt = random(360);
	my.roll = random(360);
	
	my.shadow = on;
	
	vec_scale(my.scale_x, random(3) + 0.3);
	
	//	my.z += my.scale_x * my.scale_x;
	
	my.material = mat_stone1;
}

function add_stones(n)
{
	var count;
	var count_max;
	
	count_max = min(n, VEG_STONE_MAX - stone_count);
	
	var neigung;
	
	if (count_max <= 0)
	{
		return;
	}
	
	trace_mode = ignore_sprites + ignore_passents + ignore_passable;

	while (count < count_max)
	{
		//Position
		temp.x = -4700 + random(9400);
		temp.y = -4700 + random(9400);
		temp.z = 2000;
		
		//Auf Boden Tracen
		trace(temp.x, vector(temp.x, temp.y, -2000));
		
		//Neigung berechnen
		if (normal.z != 0)
		{
			neigung = atan( sqrt(normal.x*normal.x + normal.y*normal.y) / normal.z );
		}
		
		//Neigung & Hhe
		if ( ((normal.z == 0) || (neigung < 40)) && (target.z < 400) )
		{
			//Grassprite erstellen
			ent_create(strStone1, target.x, aStone1);
			
			//Graszhler weiter
			stone_count += 1;
			count += 1;
		}
		
	}
}

//////////////////////////////
// Allgemeine Einstellungen / Umgebungsvariablen
//////////////////////////////

//Allgemein
starter init
{
	randomize();
	
	//Bildeinstellungen
	camera.arc = 70;
	
	//Auflsung
	var _w;
	var _r;
	
	//Standard
	_w = 1;
	_r = 6;
	
	//Fenster / Fullscreen
	ifdef fs;
		_w = 1;
	ifelse;
		_w = 2;
	endif;
	
	//Auflsung
	ifdef res1;
		_r = 6; //640 x 480
	ifelse;
		ifdef res2;
			_r = 7; //800 x 600
		ifelse;
			ifdef res3;
				_r = 8; //1024 x 768
			ifelse;
				ifdef res4;
					_r = 9; //1280 x 960
				ifelse;
					ifdef res5;
						_r = 11; //1600 x 1200;
					endif;
				endif;
			endif;
		endif;
	endif;
	
	video_switch(_r, 0, _w);
}

//////////////////////////////
// Sequenzen
//////////////////////////////
//////////////////////////////
// 1. Level
//////////////////////////////
string level1_str = <1.wmb>;
var loaded;

//Einstellungen fr 1. Sequenz
sky clouds1
{
	type = <clouds1.tga>;
	layer = 1;
	
	flags = dome;
}

sky scene1
{
	type = <scene1.tga>;
	layer = 2;	// verdecke Sky-Entities mit niedrigeren Layers
	scale_x = 0.25;	// ergibt ein Sichtfeld von 90 Grad (360 * 0.25 = 90)
	tilt = -30;	// zieht es um 10 Grad nach unten
	
	flags = scene;
}

material mat_sun1
{
	effect = "
	
	texture entSkin1;
	
	technique bright_sun
	{
		pass p0
		{
			Texture[0] = <entSkin1>;
			
			TextureFactor = 0x00FFF8E7;
			ColorArg1[0] = TFactor;
			ColorOp[0] = SelectArg1;
			
			AlphaArg1[0] = Texture;
			AlphaOp[0] = SelectArg1;
			
			ColorOp[1] = Disable;
		}
	}
	
	";
}

sky sun1
{
	type = <sun1.tga>;
	layer = 3;
	
	material = mat_sun1;
	
	x = 2800;
	y = 0;
	z = 1200;
	
	scale_x = 0.6;
	scale_y = 0.6;
}

//////////////////////////////
// Set camera above terrain + Correction
//////////////////////////////

SVEC v_05;
function _cam()
{
	while (1)
	{
		vec_set(v_05, vector(camera.x, camera.y, 2000));
		trace_mode = ignore_sprites + ignore_passable + ignore_passents;
		trace(v_05, vector(v_05.x, v_05.y, -2000));
		
		if (camera.z < (target.z + 40))
		{
			camera.z += target.z - camera.z + 40;
//			vec_add(camera.x, normal.x);
		}
		
		wait(1);
	}
}

function init_environment1()
{
	sleep(1);
	level_load(level1_str);

	wait(3);
	//Nebel und Hintergrundfarbe
	vec_set(bg_color.blue, vector(189, 179, 164));
	vec_set(d3d_fogcolor1.blue, bg_color.blue);
	ifndef nofog;
		fog_color = 1;
		camera.fog_start = 2000;
		camera.fog_end = 3000;
	endif;
	camera.clip_far = 3400;
	camera.clip_near = 0;
	//Kamerakorrektion
	_cam();
	
	//Himmel
	sky_clip = 20;
	clouds1.visible = on;
	scene1.visible = on;
	sun1.visible = on;
	
	sun_angle.pan = 0;
	sun_angle.tilt = 20;
	
	
	//Vegetation
	//Steine / Gras / Holz
	ifdef detail0;
	ifelse;
		ifdef detail1;
			ifdef foilage;
				add_grass(2000, MODE_CROWDS);
			endif;
			add_stones(20);
			add_wood(30);
		ifelse;
			ifdef detail2;
				ifdef foilage;
					add_grass(3000, MODE_CROWDS);
					add_grass(1000, MODE_COVERALL);
				endif;
				add_stones(50);
				add_wood(60);
			endif;
		endif;
	endif;	

	//loaded
	ifndef run;
		loaded = 0;
	ifelse;
		loaded = 1;
	endif;
}
/*
//Einstellungen fr 2. Sequenz
//////////////////////////////
// 2. Level
//////////////////////////////
string level2_str = <2.wmb>;

function init_environment2()
{
	sleep(1);
	level_load(level2_str);

	wait(1);
	//Nebel und Hintergrundfarbe
	vec_set(bg_color.blue, vector(195, 212, 229));
	vec_set(d3d_fogcolor1.blue, bg_color.blue);
	fog_color = 1;
	camera.fog_start = 2000;
	camera.fog_end = 3000;
	camera.clip_far = camera.fog_end;
	camera.clip_near = 0;
	
	//Himmel
	sky_clip = 20;
	clouds1.visible = on;
	scene1.visible = on;
	
	//Vegetation
	//	toggle_grass();

	//loaded
	loaded = 1;
}
*/

//Debug Panel
ifdef develop;

	panel pDebug1
	{
		//	bmap = blendmap2;
		pos_x = 100;
		pos_Y = 0;
		layer = 20;
		red = 255;
		green = 64;
		blue = 0;
		
		digits = 10, 10, 4.0, ftDebug, 1, grass_count;
		digits = 10, 30, 4.0, ftDebug, 1, stone_count;
		digits = 10, 50, 4.0, ftDebug, 1, wood_count;
		
		flags = d3d, refresh, visible;
	}

endif;