//////////////////////////////
// shader.wdl
//////////////////////////////
//////////////////////////////
// Winter Contest Project v2.01a, (C) 04, phosphor games, Johannes Bausch
//////////////////////////////
//////////////////////////////
// Terrainshader
//////////////////////////////


bmap blendmap1 = <1_slopemap.tga>;	// Blender for Pass p2

function mtlTerrain1_init()
{
	//
	// Adjust the Texture transform to do some tiling of the Textures
	//

	mtl.matrix11 = float(48);	// default u scale 
	mtl.matrix22 = float(48);	// default v scale

}

material mtlTerrain1
{
	ambient_red = 20;
	ambient_blue = 20;
	ambient_green = 20;
	
	power = 50;
	
	skin1 = blendmap1;

	event = mtlTerrain1_init;
	
	effect = 
	"
	matrix matMtl;
	
	Texture mtlSkin1;	// Blending map for Pass P1
	
	Texture entSkin1;	// Tileable Tex for Pass P0
	Texture entSkin2;	// Tileable Tex for Pass P2
	
	technique ffp_terrain_tex
	{
		pass P0
		{
			
			AlphaBlendEnable = False;
			AlphaTestEnable = False;
			
			ZEnable = True;
			ZWriteEnable = True;
			
			////////////////////////////////////////
			// Stage 0
			///////////////////////////////////////

			Texture[0] = <entSkin1>;	

			magFilter[0] = Linear;
			minFilter[0] = Linear;
			mipFilter[0] = Linear;
			
			TextureTransformFlags[0] = Count2;
			TextureTransform[0] = <matMtl>;
			TexCoordIndex[0] = 0;

			ColorArg1[0] = Texture; 
			ColorOp[0] = Modulate2x;
			ColorArg2[0] = Diffuse;
			
			AlphaOp[0] = Disable;
			ColorOp[1] = Disable;
		}

		pass P1
		{
			
			//////////////////////////////////////
			// Stage 0 - Setup the alphamap
			//////////////////////////////////////            		

			// We need to enable alphablending 
			
			AlphaBlendEnable = True;
			SrcBlend = SrcAlpha;
			DestBlend = InvSrcAlpha;
			
			AlphaTestEnable = True;
			AlphaRef = 0x00000000;      	//Reference Value
			AlphaFunc = Greater;  	  	//Comparison Function
			
//			DepthBias = 1;      			// To help with depth testing in this pass		
			
			Texture[0] = <mtlSkin1>;

			magFilter[0] = Linear;
			minFilter[0] = Linear;
			mipFilter[0] = Linear;
			
			//
			// We don't want the alphamap to tile
			//
			TextureTransformFlags[0] = Disable;
			TexCoordIndex[0] = 0;	
			
			ColorArg1[0] = Current; 
			ColorOp[0] = SelectArg1;
			AlphaArg1[0] = Texture | AlphaReplicate;
			AlphaOp[0] = SelectArg1;

			//////////////////////////////////////////////
			// Stage 1 -- This is the Texture to splat
			//////////////////////////////////////////////
			Texture[1] = <entSkin2>;

			// ZEnable is supposed to default to True.  
			// This enables depth testing to determine what pixels 
			// are visible to the viewer
			
			ZEnable = True;
			ZWriteEnable = True;

			magFilter[1] = Linear;
			minFilter[1] = Linear;
			mipFilter[1] = Linear;
			
			TextureTransformFlags[1] = Count2;
			TextureTransform[1] = <matMtl>;
			TexCoordIndex[1] = 1;

			ColorArg1[1] = Texture; 
			ColorOp[1] = Modulate2x;
			ColorArg2[1] = Diffuse;
			
			AlphaArg1[1] = Current | AlphaReplicate;

			AlphaOp[1] = Disable;
			ColorOp[2] = Disable;
		}
	} // End of Technique
	
/*	technique f1 //Fallback
	{
		pass p0
		{
			Texture[0] = <entSkin1>;
			
			TextureTransformFlags[0] = Count2;
			TextureTransform[0] = <matMtl>;
			TexCoordIndex[0] = 1;
			
			ColorArg1[0] = Texture;
			ColorOp[0] = Modulate2x;
			ColorArg2[0] = Diffuse;
			
			AlphaOp[0] = Disable;
			ColorOp[1] = Disable;
		}
	}
*/
	"; // End of the effect string

} // End of Material

material mtlTerrain2
{
	ambient_red = 20;
	ambient_blue = 20;
	ambient_green = 20;
	
	power = 50;
	event = mtlTerrain1_init;

	effect = "
	texture entSkin1;
	matrix matMtl;
	
	technique f1 //Fallback
	{
		pass p0
		{
			Texture[0] = <entSkin1>;
			
			TextureTransformFlags[0] = Count2;
			TextureTransform[0] = <matMtl>;
			TexCoordIndex[0] = 1;
			
			ColorArg1[0] = Texture;
			ColorOp[0] = Modulate2x;
			ColorArg2[0] = Diffuse;
			
			AlphaOp[0] = Disable;
			ColorOp[1] = Disable;
		}
	}
	";	
}

